-----------------------------------------------------------------------------------------
-- Trigonometry
-----------------------------------------------------------------------------------------

--Return the direction (in degrees) to a point.
function point_direction(x1,y1,x2,y2)
	return math.deg(math.atan2((y1-y2),(x1-x2)));
end

--Return the distance to a point.
function point_distance(x1,y1,x2,y2)
        return (math.sqrt(math.pow(x2-x1,2)+math.pow(y2-y1,2)));
end

--Return the x coordinate of a point dist pixels away in direction ang.
function lengthdir_x(dist,ang)
	return dist * math.cos(ang*math.pi/180);
end

--Return the y coordinate of a point dist pixels away in direction ang.
function lengthdir_y(dist,ang)
	return dist * math.sin(ang*math.pi/180);
end

-----------------------------------------------------------------------------------------
-- Grappling
-----------------------------------------------------------------------------------------

function GrappleIn(object,pointx,pointy,speed)
	local dir = point_direction(object.Pos.X,object.Pos.Y,pointx,pointy);
	object.Vel = Vector(object.Vel.X - lengthdir_x(speed,dir),object.Vel.Y - lengthdir_y(speed,dir) - (SceneMan.GlobalAcc.Y * TimerMan.DeltaTimeSecs) * 0.9);
end

-----------------------------------------------------------------------------------------
-- Object code
-----------------------------------------------------------------------------------------

function Create(self)
	--Grappled or not?
	self.grappled = false;
	
	--The grappling hook.
	self.hook = nil;
	
	--Force to shoot out hook with.
	self.hookforce = 50;
	
	--Force to reel in with.
	self.reelforce = 5;
	
	--Radius of each segment of the hook.
	self.segrad = 5;
end

function Update(self)
	--Toggle grappling.
	if self:GetController():IsState(Controller.BODY_JUMPSTART) then
		if self.grappled then
			self.grappled = false;
		else
			self.grappled = true;
			--Spawn the hook and fling it.
			self.hook = CreateMOSRotating("Crobotech.rte/Grappling Hook");
			self.hook.Pos = self.Pos;
			local angle = -self:GetAimAngle(true);
			self.hook.Vel = Vector(self.hookforce*math.cos(angle),self.hookforce*math.sin(angle));
			MovableMan:AddParticle(self.hook);
		end
	end
	
	--Grappling stuff.
	if MovableMan:IsParticle(self.hook) and self.grappled then
		--If the hook has hit something...
		if self.hook.PinStrength > 0 then
			--Get the distance from the target.
			local dist = point_distance(self.Pos.X,self.Pos.Y,self.hook.Pos.X,self.hook.Pos.Y);
			--Home in toward target.
			GrappleIn(self,self.hook.Pos.X,self.hook.Pos.Y,self.reelforce*dist/5000);
			--Create the rope.
			local dir = math.rad(point_direction(self.Pos.X,self.Pos.Y,self.hook.Pos.X,self.hook.Pos.Y))+math.pi;
			local seg = nil;
			for i=0,dist,self.segrad do
				seg = CreateMOSRotating("Crobotech.rte/Grapple Rope Segment");
				seg.Pos = self.Pos+Vector(i*math.cos(dir),i*math.sin(dir));
				MovableMan:AddParticle(seg);
			end
		end
	end
end