function Create(self)
	--How much force will be applied to the projectile.
	self.force = 30;
	
	--Upper aiming angle limit.
	self.upLimit = 1.4;
	
	--Gravity's effects.
	self.grav = SceneMan.GlobalAcc.Y * TimerMan.DeltaTimeSecs;
	
	--How many loops to skip before doing calculations (for optimization purposes).
	self.loopskip = 5;
	
	--Switched to viewing actor or not?
	self.switched = false;

	--Actor for viewing long range.
	self.viewactor = CreateActor("Target View Actor");
	self.viewactor.Team = -1;
	self.viewactor.Mass = self.ID;
	MovableMan:AddActor(self.viewactor);
end

function Update(self)
	--Restrain the weapon from going too low.
	if self.HFlipped == false and self:GetAimAngle(false) < self.RotAngle then
		self:SetAimAngle(self.RotAngle);
	elseif self.HFlipped == true and self:GetAimAngle(false) < -self.RotAngle then
		self:SetAimAngle(-self.RotAngle);
	--elseif self.HFlipped == false and self:GetAimAngle(false) > (self.RotAngle + self.upLimit) then
		--self:SetAimAngle(self.RotAngle + self.upLimit);
	--elseif self.HFlipped == true and -self:GetAimAngle(false) < (self.RotAngle - self.upLimit) then
		--self:SetAimAngle(-self.RotAngle + self.upLimit);
	end
	
	--Check if this is being used by a player.
	local isused = false;
	for player = 0,ActivityMan:GetActivity().PlayerCount do
		actor = ActivityMan:GetActivity():GetControlledActor(player);
		if actor then
			if actor.ID == self.ID then
				isused = true;
			end
		end
	end

	--Get the controller.
	local controller = self:GetController();
	
	--Search for the gun.
	local gun = nil;
	for i=1,MovableMan:GetMOIDCount()-1 do
		local part = ToMovableObject(MovableMan:GetMOFromID(i));
		if part.PresetName == "Crobotech Tank Gun" and part.RootID == self.ID then
			gun = ToHDFirearm(part);
		end
	end
	--If the gun was found...
	if gun ~= nil then
		if self:IsPlayerControlled() == true then
			--Use this to count the number of loops.  We can reduce lag by skipping calculations with this.
			local i = 0;

			--This will be updated with velocity during the drawing loop.
			local pos = Vector(gun.Pos.X,gun.Pos.Y);
			
			--The last position before loops were skipped.
			local lastpos = Vector(pos.X,pos.Y);
			
			--The projected velocity of the particle.
			local projvel = Vector(self.force * math.cos(-self:GetAimAngle(true)),self.force * math.sin(-self:GetAimAngle(true)));

			--Keeps track of whether we've hit ground yet or not.
			local hitground = false;

			--While the position is off the ground...
			while hitground == false and i < 500 do
				--Apply gravity.
				projvel.Y = projvel.Y + (self.grav);
				
				--Add the speed.
				pos = pos + projvel / 3;
				
				--Make sure the dot is not past the looping point of the map.  If it is, change it to not be.
				if pos.X > SceneMan.SceneWidth then
					pos.X = pos.X % SceneMan.SceneWidth;
				end
				
				if pos.X < 0 then
					pos.X = pos.X + SceneMan.SceneWidth;
				end

				if i % self.loopskip == 0 then
					--Put particle in place.
					local dot = CreateMOPixel("Target Dots Glow");
					dot.Pos = pos;
					MovableMan:AddParticle(dot);
					
					--Update the last position.
					lastpos = Vector(pos.X,pos.Y);
				end

				--Check if we've hit ground.
				if SceneMan:GetTerrMatter(pos.X,pos.Y) ~= 0 then
					--Flag that we've hit ground.
					hitground = true;
					--Place a particle at this spot so we know where the explosion will happen.
					local hitpos = SceneMan:MovePointToGround(pos,0,0);
					local target = CreateMOPixel("Target Aimer Glow");
					target.Pos = hitpos;
					MovableMan:AddParticle(target);
					--Update the view actor's position.
					self.viewactor.Pos = hitpos;
					--Move the player's view to this position if sharp aim is hit.
					if isused == true and controller:IsState(Controller.AIM_SHARP) then
						local player = controller.Player;
						ActivityMan:GetActivity():SwitchToActor(self.viewactor,controller.Player,-1);
						self:SetControllerMode(Controller.CIM_PLAYER,player);
						self.switched = true;
					end
				end
				
				--Increment loop counter.
				i = i + 1;
			end
		else
			local brain = nil;
			if self.Team == 0 then
				local brain = ActivityMan:GetActivity():GetPlayerBrain(Activity.TEAM_2);
			else
				local brain = ActivityMan:GetActivity():GetPlayerBrain(Activity.TEAM_1);
			end
			
			if MovableMan:IsActor(brain) and ActivityMan:GetActivity():PlayersInTeamCount(self.Team) == 0 then
				if hitpos.X < brain.Pos.X then
					if self.HFlipped == true then
						self:SetAimAngle(self:GetAimAngle(false) + 0.1);
					else
						self:SetAimAngle(self:GetAimAngle(false) - 0.1);
					end
				elseif hitpos.X > brain.Pos.X then
					if self.HFlipped == true then
						self:SetAimAngle(self:GetAimAngle(false) - 0.1);
					else
						self:SetAimAngle(self:GetAimAngle(false) + 0.1);
					end
				end
			end
		end
	end
	
	--If the player switched away from the viewing actor, set this back to AI mode.
	if self.switched == true and self.viewactor:IsPlayerControlled() == false then
		self.switched = false;
		--If no players are controlling this either, turn on AI.
		local actor = nil;
		
		if isused == false then
			self:SetControllerMode(Controller.CIM_AI,-1);
		end
	end
end

function Destroy(self)
	self.viewactor.ToDelete = true;
end